/*
** File: trfmt.c
**
** Description: Bramshill transcription formatter.
**              This program acts as a filter to reformat a Bramshill
**              transcription. The transcription file is copied from
**              standard input to standard output. The optional argument
**		specifies the number of characters per line (default 80).
**
**		For example, to prepare a listing with maximum line length
**		of 72 characters, you might use a command like---
**
**		trfmt 72 <S1234.TMT >S1234.LIS
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define DEFLINE 80
#define MINLINE 24
#define MAXLINE 256
#define LM 11           /* left margin for continued lines */

/* function to parse command line */
void clparse(argc, argv, llen)
    int argc;
    char **argv;
    int *llen;
{
    if (argc > 2)
    {
	fprintf(stderr, "Bad command, use: trfmt [line length]\n");
	exit(1);
    }
    if (argc == 2)
    {
	if (sscanf(argv[1], "%d", llen))
	{
	    if (*llen < MINLINE || *llen > MAXLINE)
	    {
		fprintf(stderr, "Bad line length, must be in range %d to %d\n",
			MINLINE, MAXLINE);
		exit(1);
	    }
	}
	else
	{
	    fprintf(stderr, "Bad integer argument %d\n", argv[1]);
	    exit(1);
	}
    }
    else
	*llen = DEFLINE;
}

/* printhdr - reads and prints header */
void printhdr()
{
    char item[12];
    
    if (!fscanf(stdin,"Transcription of Bramshill item %s\n", item))
    {
	fprintf(stderr, "Error reading item header\n");
	exit(1);
    }
    printf(" Bramshill Item %s\n", item);
    printf(" Start Len Text\n");
    printf(" ----- --- ------\n");
}
/* getutt - read an utterance, returns 1 if successful, 0 if end of file */
int getutt(buf, bufsize)
    char *buf;
    int bufsize;
{
    int c;
    while ((c = getchar()) != '\n')
    {
	if (c == EOF)
	{
	    if (ferror(stdin))
	    {
		perror("getc");
		exit(1);
	    }
	    else
		return 0;
	}
        if (bufsize > 1)
	{
	    *buf++ = c;
	    bufsize--;
	}
	else
	{
	    fprintf(stderr, "Buffer overflow, utterance too long\n");
	    exit(1);
	}
    }
    *buf = '\0';
    return 1;
}

/* pututt - output an utterance */
void pututt(u, lmax)
    char *u;
    int lmax;
{
    char *p;
    int ccount = 0;
    int s;
    int l;
    int n;
    
    /* first extract timings and convert from 0.1s to 1s units */
    
    sscanf(u, "%d %d ", &s, &l);
    printf("%6.1f %3.1f", (float)s / 10.0,  (float)l / 10.0);
    
    /*
    ** then deal with the rest of the utterance, lexical word by word,
    ** We skip the first two words because they are already output.
    */
    n = 1;
    ccount = LM;
    while (*u)
    {
	p = strchr(u, ' ');      /* locate next space ... */
	if (p == NULL)
	    p = strchr(u, '\0'); /* .. or terminator      */
	if (n > 2)
	{
	    /* test to see if there is room on the line */
	    if ((lmax - ccount - 1) > (p - u))
	    {
		/* there's room so just output a space */
		
		putchar(' ');
		ccount++;
	    }
	    else
	    {
		/* no room, so output newline and indent */
		int i;
		
		putchar('\n');
		for (i = 0; i < LM; i++)
		    putchar(' ');
		ccount = LM;
	    }
	    while (u < p)
	    {
		putchar(*u++);
		ccount++;
	    }
	}
        if (*p != NULL)
            u = p+1;
	n++;
    }
    putchar('\n');
}

/* main program */
main(argc, argv)
    int argc;
    char **argv;
{
    int llen;
    char uttbuf[MAXLINE + 1];
    
    /* get command line argument */
    clparse(argc, argv, &llen);
    
    /* print the header */
    printhdr();
    /* reformat each utterance in turn */
    while (getutt(uttbuf, sizeof(uttbuf)))
	pututt(uttbuf, llen);
    exit(0);
}
